import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:model_of_the_times/requester/requester.dart';
import 'package:model_of_the_times/utils/toast.dart';
import 'package:model_of_the_times/views/common/data_required.dart';
import 'package:model_of_the_times/views/common/full_paged_struct.dart';

class ActivityCard extends StatefulWidget {
  final String picPath;
  final int id;
  final String startDate;
  final String endDate;
  final int status;
  final String title;
  final String content;
  final String sponsor;
  final int signUpNum;
  const ActivityCard({super.key, required this.picPath, required this.id, required this.startDate, required this.endDate, required this.status, required this.title, required this.content, required this.sponsor, required this.signUpNum});

  @override
  State<ActivityCard> createState() => _ActivityCardState();
}

class _ActivityCardState extends State<ActivityCard> {
  @override
  Widget build(BuildContext context) {
    return Card(
      child: Padding(padding: const EdgeInsets.all(10), child: Column(
        children: [
          Image.network(resolve(widget.picPath).toString(), height: 100, fit: BoxFit.fitHeight),
          ListTile(
            title: Text(widget.title),
            subtitle: Column(
              children: [
                Text("活动时间：${widget.startDate} - ${widget.endDate}"),
                Text("发起方：${widget.sponsor}"),
                Text("简介：${widget.content}", maxLines: 1, overflow: TextOverflow.ellipsis)
              ],
            ),
          ),
          const Divider(),
          Flex(
            direction: Axis.horizontal,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Flex(
                direction: Axis.horizontal,
                children: [
                  const Icon(Icons.add),
                  Text("已报名${widget.signUpNum}人")
                ],
              ),
              OutlinedButton(onPressed: (){
                if (widget.status == 1) {
                  Navigator.push(context, MaterialPageRoute(builder: (context) => ActivityDetails(id: widget.id)));
                }
              }, child: Text(widget.status == 1 ? "去报名" : widget.status == 2 ? "报名截止" : "已报名"))
            ],
          )
        ],
      )),
    );
  }
}



class Activity extends StatefulWidget {
  const Activity({super.key});

  @override
  State<Activity> createState() => _ActivityState();
}

class _ActivityState extends State<Activity> {
  @override
  Widget build(BuildContext context) {
    return DataRequired(
      fetchData: (global) async {
        var req = global.requester;
        var response = await req.get(resolve("/activity/app-o/list"));
        return jsonFromResponse(response);
      },
      afterLoading: (data) {
        if (kDebugMode) {
          print(data);
        }
        data['rows'].sort((a,b){return (a['status'] - b['status'] )as int;});
        return ListView(
          children: data['rows'].map<Widget>((e) {
            return Container(
              margin: const EdgeInsets.only(top: 10),
              child: ActivityCard(picPath: e['picPath'], id: e['id'], startDate: e['startDate'], endDate: e['endDate'], status: e['status'], title: e['title'], content: e['content'], sponsor: e['sponsor'], signUpNum: e['signUpNum']),
            );
          }).toList(),
        );
      }
    );
  }
}

class ActivityDetails extends StatefulWidget {
  final int id;
  const ActivityDetails({super.key, required this.id});

  @override
  State<ActivityDetails> createState() => _ActivityDetailsState();
}

class _ActivityDetailsState extends State<ActivityDetails> {
  @override
  Widget build(BuildContext context) {
    return FullPagedStruct(
      title: "活动详情",
      child: DataRequired(
        fetchData: (global) async {
          return jsonFromResponse(await global.requester.get(resolve("/activity/app-o/detail?id=${widget.id}")));
        },
        afterLoading: (data) {
          data = data['data'];
          return Stack(
            children: [
              ListView(
                children: [
                  Image.network(resolve(data['picPath']).toString()),
                  ListTile(
                    title: Text(data['title']),
                    subtitle: Column(
                      children: [
                        Text("活动时间：${data['startDate']} - ${data['endDate']}"),
                        Text("报名截止时间：${data['signUpEndDate']}"),
                        Text("发起方：${data['sponsor']}"),
                      ],
                    ),
                  ),
                  const Divider(),
                  Flex(
                    direction: Axis.vertical,
                    children: [
                      const Flex(
                        direction: Axis.horizontal,
                        children: [
                          SizedBox(
                            height: 20,
                            child: VerticalDivider(thickness: 8),
                          ),
                          Text("活动详情"),
                        ],
                      ),
                      Text(data['content'])
                    ],
                  )
                ],
              ),
              Positioned.fill(
                bottom: 0,
                child: Flex(
                  direction: Axis.horizontal,
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  crossAxisAlignment: CrossAxisAlignment.end,
                  children: [
                    Flex(
                      direction: Axis.horizontal,
                      children: [
                        Text(data['signUpNum'].toString()),
                        const Text("人已报名")
                      ],
                    ),
                    TextButton(
                      onPressed: () async {
                        var data = jsonFromResponse(await GlobalInformation.getInstance().requester.get(resolve("/activity/app/signUp?id=${widget.id}")));
                        if (kDebugMode) {
                          print(data);
                        }
                        if (data['code'] == 200){
                          if (context.mounted){
                            Navigator.pop(context);
                            return;
                          }
                        }
                        if (context.mounted){
                          toast("报名失败，请重试！", context);
                        }
                      },
                      child: const Text("报名")
                    )
                  ],
                ),
              )
            ],
          );
        }
      )
    );
  }
}
